# Generated by roxygen2: do not edit by hand

export(AIRRSchema)
export(DataFileSchema)
export(GenotypeSetSchema)
export(GermlineSetSchema)
export(InfoSchema)
export(RearrangementSchema)
export(RepertoireSchema)
export(load_schema)
export(read_airr)
export(read_rearrangement)
export(read_tabular)
export(validate_airr)
export(validate_rearrangement)
export(validate_tabular)
export(write_airr)
export(write_rearrangement)
export(write_tabular)
exportClasses(Schema)
exportMethods("$")
exportMethods("[")
exportMethods(names)
import(methods)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(readr,col_character)
importFrom(readr,col_double)
importFrom(readr,col_integer)
importFrom(readr,col_logical)
importFrom(readr,cols)
importFrom(readr,read_tsv)
importFrom(readr,write_tsv)
importFrom(stats,setNames)
importFrom(stringi,stri_trim)
importFrom(tools,file_ext)
importFrom(yaml,read_yaml)
importFrom(yaml,write_yaml)
