AIRR Standards Reference Library
-------------------------------------------------------------------------------

`airr` is an R package for working with data formatted according to the AIRR 
Standards schemas. It includes the full set of schema definitions
along with simple functions for read, write and validation.
