#' @import      methods
#' @importFrom  readr     read_tsv write_tsv 
#'                        cols col_character col_logical col_integer col_double
#' @importFrom  stats     setNames
#' @importFrom  stringi   stri_trim
#' @importFrom  jsonlite  fromJSON toJSON
#' @importFrom  tools     file_ext
#' @importFrom  yaml      read_yaml write_yaml
NULL
